#ifndef H__CLUSTERFRAMEWORK__H
#define H__CLUSTERFRAMEWORK__H

#include "OGRE\Ogre.h"

#include <OgreCamera.h>
#include <OgreEntity.h>
#include <OgreLogManager.h>
#include <OgreRoot.h>
#include <OgreViewport.h>
#include <OgreSceneManager.h>
#include <OgreRenderWindow.h>
#include <OgreConfigFile.h>
 
#include <OISEvents.h>
#include <OISInputManager.h>
#include <OISKeyboard.h>
#include <OISMouse.h>
 
#include <SdkTrays.h>
#include <SdkCameraMan.h>

class ClusterGeometry
{
public:
	bool mVisibility;
	virtual ~ClusterGeometry(){};
	virtual bool isLeaf() = 0;
	void setVisibility( bool v ) { mVisibility = v; };
};

class ClusterNode: public ClusterGeometry
{
public:
	ClusterGeometry* leftChild;
	ClusterGeometry* rightChild;
	bool isLeaf() { return false; };
};

class ClusterLeaf: public ClusterGeometry
{
	//Here you may need some sort of list of vertices or triangles..
	bool isLeaf() { return true; };
};
 
class ClusterFramework : public Ogre::FrameListener, public Ogre::WindowEventListener, public OIS::KeyListener, public OIS::MouseListener, OgreBites::SdkTrayListener
{
public:
    ClusterFramework(void);
    virtual ~ClusterFramework(void);
    bool configureOgre(void);
protected:
    Ogre::Root *mRoot;
    Ogre::Camera* mCamera;
    Ogre::SceneManager* mSceneMgr;
    Ogre::RenderWindow* mWindow;
    Ogre::String mResourcesCfg;
    Ogre::String mPluginsCfg;
 
	Ogre::OverlaySystem *mOverlaySystem;
 
    // OgreBites
	OgreBites::InputContext mInputContext;
    OgreBites::SdkTrayManager* mTrayMgr;
    OgreBites::SdkCameraMan* mCameraMan;      // basic camera controller
    OgreBites::ParamsPanel* mDetailsPanel;    // sample details panel
    bool mCursorWasVisible;                   // was cursor visible before dialog appeared
    bool mShutDown;
 
    // OIS Input devices
    OIS::InputManager* mInputManager;
    OIS::Mouse*    mMouse;
    OIS::Keyboard* mKeyboard;
 
    // Ogre::FrameListener
    virtual bool frameRenderingQueued(const Ogre::FrameEvent& evt);
 
    // OIS::KeyListener
    virtual bool keyPressed( const OIS::KeyEvent &arg );
    virtual bool keyReleased( const OIS::KeyEvent &arg );
    // OIS::MouseListener
    virtual bool mouseMoved( const OIS::MouseEvent &arg );
    virtual bool mousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id );
    virtual bool mouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id );
 
    // Ogre::WindowEventListener
    virtual void windowResized(Ogre::RenderWindow* rw);
    virtual void windowClosed(Ogre::RenderWindow* rw);

	void draw3DLine(std::string name, Ogre::Vector3 from, Ogre::Vector3 to, std::string materialName);

	//Your tasks
	void prepareModel();
	ClusterGeometry* cluster(Ogre::EdgeData* modelData, int amountOfClusters, Ogre::Vector3 cameraPosition, Ogre::Vector3 viewDirection);
	bool facing(ClusterGeometry* cg, double alpha, Ogre::Vector3 cameraPosition, Ogre::Vector3 viewDirection);
};
 
#endif // #ifndef H__CLUSTERFRAMEWORK__H